!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
!
! !MODULE: gmaopert_Mod
! 
! !DESCRIPTION: 
!   This module contains routines to peturb either the forcing,
!   prognostic variables, or observations based on the algorithmic
!   implementations of Rolf Reichle (NASA GMAO). 
!   
! !REVISION HISTORY: 
!  08Jul05    Sujay Kumar; Initial Specification
!  24Nov10    Sujay Kumar; Added support for time varying 
!                          perturbations
!EOP
module gmaopert_Mod
  use ESMF
  use landpert_module

  implicit none
  PRIVATE
!------------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!------------------------------------------------------------------------------
  PUBLIC :: gmaoPert_init
  PUBLIC :: gmaoPert_setup
  PUBLIC :: gmaoperturb
  PUBLIC :: gmaopert_readrestart
  PUBLIC :: gmaopert_writerestart

  type forcpertdec
     type(pert_param_type), dimension(:),pointer :: forcepert_param
     integer, dimension(:,:), pointer :: Forcepert_rseed
     real, dimension(:,:,:,:), pointer:: Forcepert
     real, dimension(:,:,:,:), pointer:: Forcepert_ntrmdt
  end type forcpertdec
  
  type progpertdec
     type(pert_param_type), dimension(:),pointer :: progpert_param
     integer, dimension(:,:), pointer :: progpert_rseed
     real, dimension(:,:,:,:), pointer:: progpert
     real, dimension(:,:,:,:), pointer:: progpert_ntrmdt
  end type progpertdec

  type obspertdec
     type(pert_param_type), dimension(:),pointer :: obspert_param
     integer, dimension(:,:), pointer :: obspert_rseed
     real, dimension(:,:,:,:), pointer:: obspert
     real, dimension(:,:,:,:), pointer:: obspert_ntrmdt
  end type obspertdec

  type(forcpertdec), pointer :: forcPert(:,:)
  type(progpertdec), pointer :: progPert(:,:)
  type(obspertdec), pointer :: obsPert(:,:)  
  integer         , pointer :: nobs(:,:)
  
  contains
!BOP
! !ROUTINE: gmaopert_init
!  \label{gmaopert_init}
!
! !DESCRIPTION:
!  Initializes memory structures for specified perturbation
!  options.
! 
! !INTERFACE:      
    subroutine gmaoPert_init(index)
! !USES: 
      use LIS_coreMod, only : LIS_rc

! !ARGUMENTS: 
      integer, intent(in) :: index
!EOP
      
      if(index.eq.1) then          
         allocate(forcPert(LIS_rc%nnest, 1))            
      elseif(index.eq.2) then 
         allocate(progPert(LIS_rc%nnest, LIS_rc%nperts))
      elseif(index.eq.3) then 
         allocate(obspert(LIS_rc%nnest, LIS_rc%ndas))
         allocate(nobs(LIS_rc%nnest,LIS_rc%ndas))
      endif
    end subroutine gmaoPert_init

!BOP
! !ROUTINE: gmaopert_setup
!  \label{gmaopert_setup}
!
! !INTERFACE:      
    subroutine gmaoPert_setup(index, k, Base_State, Pert_State)
! !USES:    
      use landpert_routines, only : assemble_pert_param,&
           get_init_pert_rseed, get_pert
      use LIS_coreMod, only : LIS_rc, LIS_domain,LIS_surface
      use LIS_logMod,  only  : LIS_verify
      use nr_ran2_gasdev, only : NRANDSEED
!
! !DESCRIPTION:
!  Initializes memory structures for specified perturbation
!  options.
! 
!EOP    
      implicit none
      integer                      :: index
      integer                      :: k 
      type(ESMF_State)             :: Base_State(LIS_rc%nnest)
      type(ESMF_State)             :: Pert_State(LIS_rc%nnest)
      integer                      :: gid
      integer                      :: status
      integer, parameter           :: N_domain = 1
      character*100                :: sname
      integer                      :: objcount
      integer, allocatable             :: init_Pert_rseed(:,:)
      integer, allocatable             :: ens_id(:)
      integer, dimension(N_domain) :: domain_id
      character*100,       allocatable :: pertobjs(:)
      type(ESMF_Field),    allocatable :: pertField(:)
      real,    allocatable             :: std(:,:,:) 
      real,    allocatable             :: std1(:)
      integer :: i,t,m,tindex,col,row
      integer :: n, nest
      real :: dx, dy, dtstep
      
      call ESMF_StateGet(Base_State(1),name=sname, rc=status)
      call LIS_verify(status, &
           "ESMF_StateGet failed in gmaopert_setup")

      if(index.eq.1) then 
            
         do nest=1,LIS_rc%nnest
            allocate(init_Pert_rseed(LIS_rc%nensem(nest),N_domain))
            allocate(ens_id(LIS_rc%nensem(nest)))
            
            dx = LIS_rc%gridDesc(nest,9)
            dy = LIS_rc%gridDesc(nest,10)
            dtstep = LIS_rc%ts
            ! allocate substructures only if needed. 
            nullify(forcPert(nest,k)%forcepert_param)
            call assemble_pert_param(Pert_State(nest), &
                 LIS_rc%lnc(nest),LIS_rc%lnr(nest),&
                 LIS_rc%nforcepert,forcPert(nest,k)%forcepert_param)
               
            allocate(forcPert(nest,k)%Forcepert_rseed(NRANDSEED,&
                 LIS_rc%nensem(nest)))
            allocate(forcPert(nest,k)%Forcepert_ntrmdt(LIS_rc%nforcepert,&
                 LIS_rc%lnc(nest),LIS_rc%lnr(nest),LIS_rc%nensem(nest)))
            allocate(forcPert(nest,k)%Forcepert(LIS_rc%nforcepert,&
                 LIS_rc%lnc(nest),LIS_rc%lnr(nest),LIS_rc%nensem(nest)))
            ! initialize just in case (should not be needed)      
            forcPert(nest,k)%Forcepert_ntrmdt = 0.    
            forcPert(nest,k)%Forcepert        = 0.   
            
            do n=1,LIS_rc%nensem(nest)
               ens_id(n) = n-1
            end do
            
            do n=1,N_domain
               domain_id(n) = n
            end do
            ! get different negative integer for each ensemble member and
            ! each domain
                  
            call get_init_Pert_rseed(LIS_rc%nensem(nest), &
                 N_domain, ens_id, domain_id, &
                 init_Pert_rseed )
            
            ! initialize first row of Forcepert_rseed (for first domain)
            
            forcPert(nest,k)%Forcepert_rseed(1,:) = init_Pert_rseed( :, 1)
            ! initial call to get_pert  (for first domain)
            !
            ! this initializes Forcepert_ntrmdt and the rest of Forcepert_rseed 
            !
            ! NOTE: after initial call to get_pert use restart files 
            !       for Forcepert_rseed and Forcepert_ntrmdt to continue
            !       the perturbation time series whenever the land model
            !       integration is interrupted

            call ESMF_StateGet(Pert_State(nest), itemCount=objcount,rc=status)
            call LIS_verify(status, &
                 "ESMF_StateGet failed in gmaopert_setup")
            
            allocate(pertobjs(objcount))
            allocate(pertField(objcount))

            call ESMF_StateGet(Pert_State(nest), itemNameList=pertobjs,&
                 rc=status)
            call LIS_verify(status,&
                 "ESMF_StateGet failed in gmaopert_setup")
      
            allocate(std(objcount,LIS_rc%lnc(nest), LIS_rc%lnr(nest)))
            allocate(std1(LIS_rc%ngrid(nest)))

            do i=1,objcount
               call ESMF_StateGet(Pert_State(nest), pertobjs(i),pertField(i),&
                    rc=status)
               call LIS_verify(status)
               
               if(LIS_rc%ngrid(nest).gt.0) then 
                  call ESMF_AttributeGet(pertField(i),"Standard Deviation",&
                       std1,rc=status)
!                       itemCount=LIS_rc%ngrid(nest),rc=status)
                  call LIS_verify(status)
               endif

               std(i,:,:) = 0.0 
               do t=1,LIS_rc%ntiles(nest)/LIS_rc%nensem(nest)            
                  do m=1, LIS_rc%nensem(nest)
                     tindex = (t-1)*LIS_rc%nensem(nest)+m
                     
                     col = LIS_domain(nest)%tile(tindex)%col
                     row = LIS_domain(nest)%tile(tindex)%row
                     gid = LIS_domain(nest)%gindex(col,row)

                     std(i,col,row) = std(i,col,row) + std1(gid)/&
                          LIS_rc%nensem(nest) 
                  enddo
               enddo
            enddo

            call get_pert(                                           &
                 LIS_rc%nforcepert, LIS_rc%nensem(nest),&
                 LIS_rc%lnc(nest),LIS_rc%lnr(nest),  &
                 dx, dy, dtstep,                                     &
                 std,                              & 
                 forcPert(nest,k)%forcepert_param, &
                 forcPert(nest,k)%Forcepert_rseed, &
                 forcPert(nest,k)%Forcepert_ntrmdt, &
                 forcPert(nest,k)%Forcepert,   &                
                 initialize_rseed=.true.,    &
                 initialize_ntrmdt=.true.            )
            
            deallocate(init_Pert_rseed)
            deallocate(ens_id)
            deallocate(std)
            deallocate(std1)
            deallocate(pertobjs)
            deallocate(pertField)
         enddo
      elseif(index.eq.2) then 
         
         do nest=1,LIS_rc%nnest
            allocate(init_Pert_rseed(LIS_rc%nensem(nest),N_domain))
            allocate(ens_id(LIS_rc%nensem(nest)))
            
            dx = LIS_rc%gridDesc(nest,9)
            dy = LIS_rc%gridDesc(nest,10)
            dtstep = LIS_rc%ts
            
            nullify(progPert(nest,k)%progpert_param)
            call assemble_pert_param(Pert_State(nest), &
                 LIS_rc%lnc(nest),LIS_rc%lnr(nest),&
                 LIS_rc%nstvars(k),progPert(nest,k)%progpert_param)
            
            allocate(progPert(nest,k)%progpert_rseed(NRANDSEED,LIS_rc%nensem(nest)))
            allocate(progPert(nest,k)%progpert_ntrmdt(LIS_rc%nstvars(k),&
                 LIS_rc%lnc(nest),LIS_rc%lnr(nest),LIS_rc%nensem(nest)))
            allocate(progPert(nest,k)%Progpert(LIS_rc%nstvars(k),LIS_rc%lnc(nest),&
                 LIS_rc%lnr(nest),LIS_rc%nensem(nest)))
            ! initialize just in case (should not be needed)      
            progPert(nest,k)%progpert_ntrmdt = 0.    
            progPert(nest,k)%progpert        = 0.   
            
            do n=1,LIS_rc%nensem(nest)
               ens_id(n) = n-1
            end do
            
            do n=1,N_domain
               domain_id(n) = n
            end do
            ! get different negative integer for each ensemble member and
            ! each domain
            
            call get_init_Pert_rseed(LIS_rc%nensem(nest),&
                 N_domain, ens_id, domain_id, &
                 init_Pert_rseed )
            
            ! initialize first row of progpert_rseed (for first domain)
            
            progPert(nest,k)%progpert_rseed(1,:) = init_Pert_rseed( :, 1)
            ! initial call to get_pert  (for first domain)
            !
            ! this initializes progpert_ntrmdt and the rest of progpert_rseed 
            !
            ! NOTE: after initial call to get_pert use restart files 
            !       for progpert_rseed and progpert_ntrmdt to continue
            !       the perturbation time series whenever the land model
            !       integration is interrupted

            call ESMF_StateGet(Pert_State(nest), itemCount=objcount,rc=status)
            call LIS_verify(status)
            
            allocate(pertobjs(objcount))
            allocate(pertField(objcount))

            call ESMF_StateGet(Pert_State(nest), itemNameList=pertobjs, rc=status)
            call LIS_verify(status)
      
            allocate(std(objcount,LIS_rc%lnc(nest), LIS_rc%lnr(nest)))
            allocate(std1(LIS_rc%ngrid(nest)))

            do i=1,objcount
               call ESMF_StateGet(Pert_State(nest), pertobjs(i),pertField(i),&
                    rc=status)
               call LIS_verify(status)
               
               if(LIS_rc%ngrid(nest).gt.0) then 
                  call ESMF_AttributeGet(pertField(i),"Standard Deviation",&
                       std1,rc=status)
!                       itemCount=LIS_rc%ngrid(nest),rc=status)
                  call LIS_verify(status)
               endif

               std(i,:,:) = 0.0 
               do t=1,LIS_rc%npatch(nest,1)/LIS_rc%nensem(nest)            
                  do m=1, LIS_rc%nensem(nest)
                     tindex = (t-1)*LIS_rc%nensem(nest)+m
                     
                     col = LIS_surface(nest,1)%tile(tindex)%col
                     row = LIS_surface(nest,1)%tile(tindex)%row
                     gid = LIS_domain(nest)%gindex(col,row)

                     std(i,col,row) = std(i,col,row) + std1(gid)/&
                          LIS_rc%nensem(nest) 
                  enddo
               enddo
            enddo
               
            call get_pert(                                           &
                 LIS_rc%nstvars(k), LIS_rc%nensem(nest),&
                 LIS_rc%lnc(nest),LIS_rc%lnr(nest),  &
                 dx, dy, dtstep,                                     &
                 std,                              & 
                 progPert(nest,k)%progpert_param, &
                 progPert(nest,k)%Progpert_rseed, &
                 progPert(nest,k)%Progpert_ntrmdt, &
                 progPert(nest,k)%Progpert,   &                
                 initialize_rseed=.true.,    &
                 initialize_ntrmdt=.true.            )
            deallocate(init_Pert_rseed)
            deallocate(ens_id)
            deallocate(std)
            deallocate(std1)
         enddo
      elseif(index.eq.3) then 
         
         do nest=1,LIS_rc%nnest
            allocate(init_Pert_rseed(LIS_rc%nensem(nest),N_domain))
            allocate(ens_id(LIS_rc%nensem(nest)))
            
            call ESMF_StateGet(Pert_State(nest), itemCount=objcount,rc=status)
            call LIS_verify(status)
            
            nobs(nest,k) = objcount
            
            dx = LIS_rc%gridDesc(nest,9)
            dy = LIS_rc%gridDesc(nest,10)
            dtstep = LIS_rc%ts
            
            nullify(obspert(nest,k)%obspert_param)
            call assemble_pert_param(Pert_State(nest), &
                 LIS_rc%lnc(nest),LIS_rc%lnr(nest),&
                 objcount,obspert(nest,k)%obspert_param)
                  
            allocate(obspert(nest,k)%obspert_rseed(NRANDSEED,&
                 LIS_rc%nensem(nest)))
            allocate(obspert(nest,k)%obspert_ntrmdt(objcount,&
                 LIS_rc%lnc(nest),LIS_rc%lnr(nest),LIS_rc%nensem(nest)))
            allocate(obspert(nest,k)%Obspert(objcount,LIS_rc%lnc(nest),&
                 LIS_rc%lnr(nest),LIS_rc%nensem(nest)))
            ! initialize just in case (should not be needed)      
            obspert(nest,k)%obspert_ntrmdt = 0.    
            obspert(nest,k)%obspert        = 0.   
            
            do n=1,LIS_rc%nensem(nest)
               ens_id(n) = n-1
            end do
            
            do n=1,N_domain
               domain_id(n) = n
            end do
            ! get different negative integer for each ensemble member and
            ! each domain
            
            call get_init_Pert_rseed(LIS_rc%nensem(nest), &
                 N_domain, ens_id, domain_id, &
                 init_Pert_rseed )
            
            ! initialize first row of obspert_rseed (for first domain)
            
            obspert(nest,k)%obspert_rseed(1,:) = init_Pert_rseed( :, 1)
            ! initial call to get_pert  (for first domain)
            !
            ! this initializes obspert_ntrmdt and the rest of obspert_rseed 
            !
            ! NOTE: after initial call to get_pert use restart files 
            !       for obspert_rseed and obspert_ntrmdt to continue
            !       the perturbation time series whenever the land model
            !       integration is interrupted
           
            call ESMF_StateGet(Pert_State(nest), itemCount=objcount,rc=status)
            call LIS_verify(status)
            
            allocate(pertobjs(objcount))
            allocate(pertField(objcount))

            call ESMF_StateGet(Pert_State(nest), itemNameList=pertobjs, rc=status)
            call LIS_verify(status)
      
            allocate(std(objcount,LIS_rc%lnc(nest), LIS_rc%lnr(nest)))
            allocate(std1(LIS_rc%ngrid(nest)))

            do i=1,objcount
               call ESMF_StateGet(Pert_State(nest), pertobjs(i),pertField(i),&
                    rc=status)
               call LIS_verify(status)
               
               if(LIS_rc%ngrid(nest).gt.0) then 
                  call ESMF_AttributeGet(pertField(i),"Standard Deviation",&
                       std1,rc=status)
!                       itemCount=LIS_rc%ngrid(nest),rc=status)
                  call LIS_verify(status)
               endif
               
               std(i,:,:) = 0.0 
               do t=1,LIS_rc%ntiles(nest)/LIS_rc%nensem(nest)            
                  do m=1, LIS_rc%nensem(nest)
                     tindex = (t-1)*LIS_rc%nensem(nest)+m
                     
                     col = LIS_domain(nest)%tile(tindex)%col
                     row = LIS_domain(nest)%tile(tindex)%row
                     gid = LIS_domain(nest)%gindex(col,row)

                     std(i,col,row) = std(i,col,row) + std1(gid)/&
                          LIS_rc%nensem(nest) 
                  enddo
               enddo
            enddo

            call get_pert(                                           &
                 objcount, LIS_rc%nensem(nest),&
                 LIS_rc%lnc(nest),LIS_rc%lnr(nest),  &
                 dx, dy, dtstep,                                     &
                 std,                              & 
                 obspert(nest,k)%obspert_param, &
                 obspert(nest,k)%Obspert_rseed, &
                 obspert(nest,k)%Obspert_ntrmdt, &
                 obspert(nest,k)%Obspert,   &                
                 initialize_rseed=.true.,    &
                 initialize_ntrmdt=.true.            )
            deallocate(init_Pert_rseed)
            deallocate(ens_id)
            deallocate(std)
            deallocate(std1)
         end do
      endif

    end subroutine gmaoPert_setup
!BOP
! !ROUTINE: gmaoperturb
!
! !INTERFACE:      
    subroutine gmaoperturb(id, n, k, Base_State, Pert_State)
! !USES:
      use LIS_coreMod, only : LIS_rc, LIS_domain,LIS_surface
      use LIS_logMod,  only  : LIS_verify
      use landpert_routines, only : get_pert
      
      implicit none
! !ARGUMENTS: 
      integer, intent(in)       :: id
      integer, intent(in)       :: n
      integer, intent(in)       :: k 
      type(ESMF_State)          :: Base_State
      type(ESMF_State)          :: Pert_State
!
! !DESCRIPTION:
!   This routine computes the perturbations (as increments) 
!   and packages them into an ESMF state. 
!   
!   The pert\_time\_type specifies whether the perturbation standard
!   deviation is constant or time varying. A value of 0 indicates
!   constant standard deviation. For pert\_time\_type = 1, a linear
!   model of perturbation standard deviation (as a linear function
!   of perturbations state in the form y=mx+b, where x is the 
!   perturbations state, m is the coefficient of standard deviation
!   and b is the base standard deviation. m and b are specified 
!   through the perturbations attributes file. 
!  
! 
!
!EOP
      integer                   :: status
      character*1               :: nestid(2)
      character*100             :: temp
      real                      :: dx, dy,dtstep
      integer                   :: t, gid,col,row,ensem, i
      integer                   :: objcount
      integer                   :: objcount_b
      character*100, pointer    :: pertobjs(:)
      character*100, pointer    :: baseobjs(:)
      type(ESMF_Field), allocatable :: pertField(:)
      type(ESMF_Field), allocatable :: baseField(:)
      real,             pointer :: pertdata1d(:)
      real,             pointer :: basedata(:)
      integer                   :: c,r,m,tindex
      real,             pointer :: pertdata2d(:,:)
      real,             allocatable :: std(:,:,:)
      real,             allocatable :: std1(:)

      write(unit=temp,fmt='(i2.2)')n
      read(unit=temp,fmt='(2a1)') nestid

!      call ESMF_StateGet(Base_State, name=sname,rc=status)
!      call LIS_verify(status)

      call ESMF_StateGet(Pert_State, itemCount=objcount,rc=status)
      call LIS_verify(status)

      call ESMF_StateGet(Base_State, itemCount=objcount_b,rc=status)
      call LIS_verify(status)

      allocate(pertobjs(objcount))
      allocate(pertField(objcount))

      allocate(baseobjs(objcount_b))
      allocate(baseField(objcount_b))

      call ESMF_StateGet(Base_State, itemNameList=baseobjs, rc=status)
      call LIS_verify(status)

      call ESMF_StateGet(Pert_State, itemNameList=pertobjs, rc=status)
      call LIS_verify(status)

      allocate(std(objcount,LIS_rc%lnc(n), LIS_rc%lnr(n)))
      allocate(std1(LIS_rc%ngrid(n)))

      do i=1,objcount
! The forcing state and perturbations state have different members. For
! lsm and obs objects, the number of elements is the same in both 
! the LSM_state (Obs_state) and LSM_pert_state (Obs_pert_state). So 
! here we need to handle these separately. 

         if(id.eq.1) then 
            baseobjs(i) = pertobjs(i)
            call ESMF_StateGet(Base_State, trim(baseobjs(i)),baseField(i),&
                 rc=status)
            call LIS_verify(status)
            call ESMF_FieldGet(baseField(i),localDE=0,farrayPtr=basedata,rc=status)
            call LIS_verify(status)

         else
            call ESMF_StateGet(Base_State, trim(baseobjs(i)),baseField(i),&
                 rc=status)
            call LIS_verify(status)
            call ESMF_FieldGet(baseField(i),localDE=0,farrayPtr=basedata,rc=status)
            call LIS_verify(status)
         endif

         call ESMF_StateGet(Pert_State, trim(pertobjs(i)),pertField(i),&
              rc=status)
         call LIS_verify(status)

         if(LIS_rc%ngrid(n).gt.0) then 
            call ESMF_AttributeGet(pertField(i),"Standard Deviation",std1,&
                 rc=status)
!                 itemCount=LIS_rc%ngrid(n),rc=status)
            call LIS_verify(status)
         endif

         if(id.eq.2) then !lsm state space
            std(i,:,:) = 0.0 
            do t=1,LIS_rc%npatch(n,LIS_rc%lsm_index)/LIS_rc%nensem(n)            
               do m=1, LIS_rc%nensem(n)
                  tindex = (t-1)*LIS_rc%nensem(n)+m
                  
                  col = LIS_surface(n,LIS_rc%lsm_index)%tile(tindex)%col
                  row = LIS_surface(n,LIS_rc%lsm_index)%tile(tindex)%row
                  gid = LIS_domain(n)%gindex(col,row)

                  std(i,col,row) = std(i,col,row) + std1(gid)/LIS_rc%nensem(n) 
               enddo
            enddo

         else 
            std(i,:,:) = 0.0 
            do t=1,LIS_rc%ntiles(n)/LIS_rc%nensem(n)            
               do m=1, LIS_rc%nensem(n)
                  tindex = (t-1)*LIS_rc%nensem(n)+m
                  
                  col = LIS_domain(n)%tile(tindex)%col
                  row = LIS_domain(n)%tile(tindex)%row
                  gid = LIS_domain(n)%gindex(col,row)
                  
                  std(i,col,row) = std(i,col,row) + std1(gid)/LIS_rc%nensem(n) 
               enddo
            enddo
         endif
      enddo
         

      if(id.eq.1) then 
         dx = LIS_rc%gridDesc(n,9)
         dy = LIS_rc%gridDesc(n,10)
         dtstep = LIS_rc%ts
         call get_pert(                                           &
              LIS_rc%nforcepert, LIS_rc%nensem(n), &
              LIS_rc%lnc(n), LIS_rc%lnr(n),       &
              dx, dy, dtstep,                  &
              std, & 
              forcPert(n,k)%forcepert_param,  &
              forcPert(n,k)%Forcepert_rseed,  &
              forcPert(n,k)%Forcepert_ntrmdt, &
              forcPert(n,k)%Forcepert)
         
         do i=1,objcount
            call ESMF_StateGet(Pert_State, pertobjs(i),pertField(i),&
                 rc=status)
            call LIS_verify(status)
            call ESMF_FieldGet(pertField(i),localDE=0,&
                 farrayPtr=pertdata1d,rc=status)
            call LIS_verify(status)

            do t=1,LIS_rc%ntiles(n)
               col = LIS_domain(n)%tile(t)%col
               row = LIS_domain(n)%tile(t)%row
               ensem = LIS_domain(n)%tile(t)%ensem
               pertdata1d(t) = forcpert(n,k)%forcepert(i,col,row,ensem)
            enddo
         enddo
      elseif(id.eq.2) then 
         dx = LIS_rc%gridDesc(n,9)
         dy = LIS_rc%gridDesc(n,10)
         dtstep = LIS_rc%ts
         call get_pert(                                           &
              objcount, LIS_rc%nensem(n), LIS_rc%lnc(n), LIS_rc%lnr(n), &
              dx, dy, dtstep,                  &
              std,                           & 
              progPert(n,k)%progpert_param,  &
              progPert(n,k)%progpert_rseed,  &
              progPert(n,k)%progpert_ntrmdt, &
              progPert(n,k)%progpert)

         do i=1,objcount
            call ESMF_StateGet(Pert_State, pertobjs(i),pertField(i),&
                 rc=status)
            call LIS_verify(status)
            call ESMF_FieldGet(pertField(i),localDE=0,&
                 farrayPtr=pertdata1d,rc=status)
            call LIS_verify(status)

            do t=1,LIS_rc%npatch(n,LIS_rc%lsm_index)
               col = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%col
               row = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%row
               ensem = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%ensem
               pertdata1d(t) = progpert(n,k)%progpert(i,col,row,ensem)
            enddo
         enddo
      elseif(id.eq.3) then 
         dx = LIS_rc%gridDesc(n,9)
         dy = LIS_rc%gridDesc(n,10)
         dtstep = LIS_rc%ts
         call get_pert(                                           &
              objcount, LIS_rc%nensem(n), LIS_rc%lnc(n), LIS_rc%lnr(n),&
              dx, dy, dtstep,                  &
              std,                           & 
              obsPert(n,k)%obspert_param,  &
              obsPert(n,k)%obspert_rseed,  &
              obsPert(n,k)%obspert_ntrmdt, &
              obsPert(n,k)%obspert)

         do i=1,objcount            
            call ESMF_StateGet(Pert_State, pertobjs(i),pertField(i),&
                 rc=status)
            call LIS_verify(status)

            call ESMF_FieldGet(pertField(i),localDE=0,&
                 farrayPtr=pertdata2d,rc=status)
            call LIS_verify(status)

            do r=1,LIS_rc%lnr(n)
               do c=1,LIS_rc%lnc(n)
                  if(LIS_domain(n)%gindex(c,r).ne.-1) then 
                     do m=1,LIS_rc%nensem(n)
                        pertdata2d(LIS_domain(n)%gindex(c,r),m) = &
                             obspert(n,k)%obspert(i,c,r,m)
                     enddo
                  endif
               enddo
            enddo
         enddo
      endif
      
      deallocate(pertobjs)
      deallocate(pertField)
      deallocate(std)
      deallocate(std1)

    end subroutine gmaoperturb


!BOP
! 
! !ROUTINE: gmaopert_writerestart
!  \label{gmaopert_writerestart}
! 
! !INTERFACE: 
    subroutine gmaopert_writerestart(n)
! !USES: 
      use LIS_coreMod,     only : LIS_rc, LIS_domain, LIS_surface,LIS_masterproc
      use LIS_historyMod,  only : LIS_writevar_restart
      use LIS_fileIOMod
      use LIS_logMod

! !ARGUMENTS: 
      implicit none

      integer, intent(in)      :: n 
! !DESCRIPTION: 
! 
! Write one single file that includes the restart info for forcing, states 
! and observations

!EOP      
      integer                  :: k, i, col, row, ensem, t
      real, allocatable            :: pertdata1d(:)
      real, allocatable            :: pertdata1d_patch(:)
      character*100            :: filen
      integer                  :: ftn 

      allocate(pertdata1d(LIS_rc%ntiles(n)))
      allocate(pertdata1d_patch(LIS_rc%npatch(n,LIS_rc%lsm_index)))

      if ( LIS_masterproc ) then

         call LIS_create_output_directory('DAPERT')         
         call LIS_create_dapert_filename(n,filen)
              
         ftn = LIS_getNextUnitNumber()
         write(LIS_logunit,*) 'Writing Perturbations restart ', trim(filen)
         open(ftn, file = trim(filen), form='unformatted')

      endif

      k = 1
      do i=1,LIS_rc%nforcepert
         
         do t=1,LIS_rc%ntiles(n)
            col = LIS_domain(n)%tile(t)%col
            row = LIS_domain(n)%tile(t)%row
            ensem = LIS_domain(n)%tile(t)%ensem
            pertdata1d(t) = forcpert(n,k)%forcepert_ntrmdt(i,col,row,ensem)
         enddo
         
         call LIS_writevar_restart(ftn,n,pertdata1d)         
      enddo
      do k=1,LIS_rc%nperts
         if(trim(LIS_rc%perturb_state(k)).ne."none") then 
            do i =1, LIS_rc%nstvars(k)
               do t=1,LIS_rc%npatch(n,LIS_rc%lsm_index)
                  col = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%col
                  row = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%row
                  ensem = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%ensem
                  pertdata1d_patch(t) =  &
                       progpert(n,k)%progpert_ntrmdt(i,col,row,ensem)
               enddo
         
               call LIS_writevar_restart(ftn,n,1,pertdata1d_patch)   
            enddo
         endif
      enddo
      
      do k=1,LIS_rc%ndas
         if(trim(LIS_rc%perturb_obs(k)).ne."none") then 
            do i=1, nobs(n,k)
               do t=1,LIS_rc%ntiles(n)
                  col = LIS_domain(n)%tile(t)%col
                  row = LIS_domain(n)%tile(t)%row
                  ensem = LIS_domain(n)%tile(t)%ensem
                  pertdata1d(t) =  obspert(n,k)%obspert_ntrmdt(i,col,row,ensem)
               enddo
               call LIS_writevar_restart(ftn,n,pertdata1d)   
            enddo
         endif
      enddo

      if(LIS_masterproc) then 
         call LIS_releaseUnitNumber(ftn)
         write(LIS_logunit,*) 'Done writing Perturbations restart ', trim(filen)
      endif
      deallocate(pertdata1d)
      deallocate(pertdata1d_patch)

    end subroutine gmaopert_writerestart

!BOP
! 
! !ROUTINE: gmaopert_readrestart
! \label{gmaopert_readrestart}
! 
! !INTERFACE: 
    subroutine gmaopert_readrestart()
! !USES: 
      use LIS_coreMod
      use LIS_logMod
      use LIS_timeMgrMod
      use LIS_historyMod
      use LIS_fileIOMod

      implicit none
! !ARGUMENTS: 

! !DESCRIPTION: 
!   This routine reads from a perturbations restart file
!EOP
      integer                   :: n 
      integer                   :: k, i,t, ftn, col, row, ensem
      integer                   :: yr,mo,da,hr,mn,ss, doy
      integer                   :: status
      character*100             :: filen
      real*8            :: time
      real              :: gmt
      real, allocatable             :: pertdata1d(:)
      real, allocatable             :: pertdata1d_patch(:)

      do n = 1, LIS_rc%nnest

         ftn = LIS_getNextUnitNumber()

         allocate(pertdata1d(LIS_rc%ntiles(n)))

         if(LIS_rc%runmode.eq."ensemble smoother") then 
            if(LIS_rc%iterationid(n).gt.1) then 
               if(LIS_rc%pertrestartInterval.eq.2592000) then
                  call ESMF_TimeGet(LIS_twStartTime,yy=yr,mm=mo,&
                       dd=da,calendar=LIS_calendar,rc=status)
                  hr = 0 
                  mn = 0 
                  ss = 0 
                  call LIS_tick(time,doy,gmt,yr,mo,da,hr,mn,ss,(-1)*1800.0)
               else
                  call ESMF_TimeGet(LIS_twStartTime,yy=yr,mm=mo,&
                       dd=da,calendar=LIS_calendar,rc=status)
                  hr = 0 
                  mn = 0 
                  ss = 0 
               endif
               call LIS_create_dapert_filename(n,filen, yr, mo, da, hr, mn, ss)
               LIS_rc%pertRestartFile(n) = filen
            endif
         endif
         
         open(ftn,file=trim(LIS_rc%pertRestartFile(n)), form='unformatted')
         write(LIS_logunit,*) 'Reading perturbations restart file ',&
              trim(LIS_rc%pertRestartFile(n))
         
         k = 1
         do i=1,LIS_rc%nforcepert
            
            call LIS_readvar_restart(ftn,n,pertdata1d)
            
            do t=1,LIS_rc%ntiles(n)
               col = LIS_domain(n)%tile(t)%col
               row = LIS_domain(n)%tile(t)%row
               ensem = LIS_domain(n)%tile(t)%ensem
               forcpert(n,k)%forcepert_ntrmdt(i,col,row,ensem) = pertdata1d(t)
            enddo
         enddo
         
         allocate(pertdata1d_patch(LIS_rc%npatch(n,LIS_rc%lsm_index)))

         do k=1, LIS_rc%nperts
            if(trim(LIS_rc%perturb_state(k)).ne."none") then 

               do i =1, LIS_rc%nstvars(k)                  
                  call LIS_readvar_restart(ftn,n,1,pertdata1d_patch)   
                  
                  do t=1,LIS_rc%npatch(n,LIS_rc%lsm_index)
                     col = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%col
                     row = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%row
                     ensem = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%ensem
                     progpert(n,k)%progpert_ntrmdt(i,col,row,ensem) = &
                          pertdata1d_patch(t) 
                  enddo
                  
               enddo
            endif
         enddo
         
         do k=1, LIS_rc%ndas
            if(trim(LIS_rc%perturb_obs(k)).ne."none") then 
               do i=1, nobs(n,k)
                  call LIS_readvar_restart(ftn,n,pertdata1d)   
                  do t=1,LIS_rc%ntiles(n)
                     col = LIS_domain(n)%tile(t)%col
                     row = LIS_domain(n)%tile(t)%row
                     ensem = LIS_domain(n)%tile(t)%ensem
                     obspert(n,k)%obspert_ntrmdt(i,col,row,ensem) = pertdata1d(t) 
                  enddo
               enddo
            endif
         enddo
         
         call LIS_releaseUnitNumber(ftn)
         write(LIS_logunit,*) 'Finished reading perturbations restart file ',&
              trim(LIS_rc%pertRestartFile(n))

         deallocate(pertdata1d)
      enddo
    end subroutine gmaopert_readrestart


end module gmaopert_Mod

